/*
 * #############################################################################
 * ATTENTION: BridgeMain must be started because of BridgeRMI (design fault) !?!
 * #############################################################################
 * 
 * @author Walther Neuper, member of the ISAC-team, 
 * Copyright (c) 2004 by Walther Neuper 
 * created Aug 30, 2004, 11:58:35 AM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to license terms.
 */
package isac.bridge;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Vector;

import org.apache.log4j.Logger;

import isac.kestore.KEStoreServices;
import isac.util.CalcChangedEvent;
import isac.util.Formalization;
import isac.util.formulae.Assumptions;
import isac.util.formulae.CalcHead;
import isac.util.formulae.CalcFormula;
import isac.util.formulae.Formula;
import isac.util.formulae.Position;
import isac.util.interfaces.ICalcElement;
import isac.util.interfaces.ICalcIterator;
import isac.util.interfaces.IToCalc;
import isac.util.interfaces.IToUser;
import isac.util.parser.FormalizationDigest;
import isac.util.tactics.Tactic;
import junit.framework.TestCase;

/**
 * @author Walther Neuper Aug 30, 2004, 11:58:35 AM
 */
public class TestBridge extends TestCase {

	static Logger logger_ = Logger.getLogger(MathEngine.class.getName());

		/**
		 * tests the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprising
	 UC
		 * 'start auto calculation' \label{SPECIFY:START:auto}
		 *
		 * see BridgeLog (sml <->java) in UC-mini-auto-step.txt
		 */
		public void testMinisubpblAutoCalc() throws Exception {
			System.out
					.println("---BEGIN isac.bridge.TestBridge#testMinisubpblAutoCalc");
	
			//prepare Formalization for start calculation in step_b
			Formalization fmzA = null, fmz = null;
			Object test = null;
			// the following would be instead reading from file ...
			//KEStoreServices ks = new
			//KEStoreServices("isac/src/java-tests/isac/kestore/data");
			//Vector vec = ks.loadContent("exp", "exp_IsacCore_Tests_1b.xml");
			//String xmlstr = (String) vec.firstElement();
			//
			//FormalizationDigest fd = new FormalizationDigest();
			//fmz = fd.parseFormalization((String) xmlstr);
			//
			File fAbroad = new File("isac/src/java-tests/isac/util/parser/data",
					"exp_IsacCore_Tests_1b.fmz");
			File f = new File(
					"isac/src/java-tests/isac/bridge/data/mini-auto-step",
					"step_a.fmz");
			try {
				ObjectInputStream oisA = new ObjectInputStream(new FileInputStream(
						fAbroad));
				ObjectInputStream ois = new ObjectInputStream(
						new FileInputStream(f));
				fmzA = (Formalization) oisA.readObject();
				fmz = (Formalization) ois.readObject();
				oisA.close();
				ois.close();
			} catch (IOException e) {
				e.printStackTrace();
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
			}
			System.out.println(".FormalizationDigest.parseFormalization");
			//System.out.println("-fmz.toString()" + fmz.toString());
			assertEquals("mini-auto: fmz from 'abroad' file ", fmzA.toSMLString(),
					fmz.toSMLString());
			System.out.println(".end step_a-----");
			//end step_a ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
			//start calculation and complete CalcHead
			MathEngine.init("localhost");
			MathEngine me = MathEngine.getMathEngine();
			IToCalc ct = me.startCalculation(fmz);
			// sent to bridge: CalcTree [(["equality (x+1=2)", "sol ...
			// sent to bridge: Iterator 1; #drop: make It.1 in CalcTree
			// sent to bridge: moveActiveRoot 1; #drop: make It.1 in CalcTree
	
			IToUser user = new MockIToUser();
			ct.addDataChangeListener(user);
			CalcChangedEvent cce = null;
	
			ct.completeCalcHead();// ######informListeners NOW!!!
			// sent to bridge: autoCalculate 1 CompleteCalcHead;
			ICalcIterator ci = ct.getActiveFormula();
			//######^^^ drop: push addDataChangeListener towards/into
			//######^^^ startCalculation and fetch ci from CalcChanged:
			//ci = (ICalcIterator) cce.getLastGeneratedFormula().clone();
			CalcHead ch = (CalcHead) ci.getFormula();
	
	// too sensible against changes in CalcHead !!!
	// f = new File("isac/src/java-tests/isac/bridge/data/mini-auto-step",
	// "step_b.calcHead");
	// //try {
	// // ObjectOutputStream oos = new ObjectOutputStream(
	// // new FileOutputStream(f));
	// // oos.writeObject(ch);
	// // oos.close();
	// //} catch (IOException e) {
	// // e.printStackTrace();
	// //}
	// try {
	// ObjectInputStream ois = new ObjectInputStream(
	// new FileInputStream(f));
	// test = (CalcHead) ois.readObject();
	// ois.close();
	// } catch (IOException e) {
	// e.printStackTrace();
	// } catch (ClassNotFoundException e) {
	// e.printStackTrace();
	// }
	// System.out.println(".calchead: " + ch.toSMLString());
	// assertEquals("mini-auto: calcHead", ch.toSMLString(), ((CalcHead) test)
	// .toSMLString());
			System.out.println(".end step_b-----");
			//end step_b
			// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
			//autoCalculate
			System.out.println(".autoCalculate");
			ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 0);
	
			cce = ((MockIToUser) user).getCalcChangedEvent();
			assertEquals("unchanged", cce.getLastUnchangedFormula().toSMLString(),
					"([],Met)");
			assertEquals("deleted", cce.getLastDeletedFormula().toSMLString(),
					"([],Met)");
			assertEquals("generated", cce.getLastGeneratedFormula().toSMLString(),
					"([],Res)");
			System.out.println(".end step_c-----");
			//end step_c
			// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
			//display the steps generated
			Object fch = null;
			ci = (ICalcIterator) cce.getLastUnchangedFormula().clone();
			int i = 0;
	// while (ci.compareTo(cce.getLastGeneratedFormula()) < 0) {
	// i++;
	// boolean b = ci.moveDown();
	// System.out.print("..iterator at " + ci.toSMLString());
	// fch = ci.getFormula();
	// System.out.println(" yields "
	// + ((ICalcElement) fch).toSMLString());
	// }
			Vector elems = ct.getElementsFromTo(cce.getLastUnchangedFormula(),
					cce.getLastGeneratedFormula(), new Integer(2), false);
			for (i=0; i<elems.size(); i++) {
				System.out.println(" yields "
				+ ((ICalcElement) elems.elementAt(i)).toSMLString());
			}
			System.out.println("mini-auto: steps= " + i);
			assertEquals("mini-auto: steps= ",i,10);
			assertEquals("mini-auto: last step",
					((ICalcElement) elems.elementAt(i-1)).toSMLString(), "[x = 1]");
			System.out.println(".end step_d-----");
			//end step_d
			// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
			logger_.debug("---END isac.bridge.TestBridge#testMinisubpblAutoCalc");
			System.out
					.println("---END isac.bridge.TestBridge#testMinisubpblAutoCalc");
		}
	
		/***************************************************************************
		 * tests the example 'IsacCore/Tests/minipbl_with_mini-subpbl' comprising
	 UC
		 * 'start interactive calculation' \label{SPECIFY:START:interactive} +
		 * \label{SOLVE:AUTO:one}}
		 *
		 * step_a like above
		 */
		public void testMinisubpblNextStep() throws Exception {
			System.out
					.println("---BEGIN isac.bridge.TestBridge#testMinisubpblNextStep");
			Formalization fmzA = null, fmz = null;
			Object test = null;
	
			//\label{SPECIFY:START:interactive}
			File f = new File(
					"isac/src/java-tests/isac/bridge/data/mini-auto-step",
					"step_a.fmz");
			try {
				ObjectInputStream ois = new ObjectInputStream(
						new FileInputStream(f));
				fmz = (Formalization) ois.readObject();
				ois.close();
			} catch (IOException e) {
				e.printStackTrace();
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
			}
			System.out.println(".end step_a-----");
			//end step_a ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
			//BridgeMain _must_ run for the following ...
			MathEngine.init("localhost");
			MathEngine me = MathEngine.getMathEngine();
	
			IToCalc ct = me.startCalculation(fmz);
	
			IToUser user = new MockIToUser();
			ct.addDataChangeListener(user);
			CalcChangedEvent cce = null;
	
			ct.completeCalcHead();
			ICalcIterator ci = ct.getActiveFormula();
			CalcHead ch = (CalcHead) ci.getFormula();
	
	// too sensible against changes in CalcHead !!!
	// f = new File("isac/src/java-tests/isac/bridge/data/mini-auto-step",
	// "step_b.calcHead");
	// try {
	// ObjectInputStream ois = new ObjectInputStream(
	// new FileInputStream(f));
	// test = (CalcHead) ois.readObject();
	// ois.close();
	// } catch (IOException e) {
	// e.printStackTrace();
	// } catch (ClassNotFoundException e) {
	// e.printStackTrace();
	// }
	// System.out.println(".calchead: " + ch.toSMLString());
	// assertEquals("mini-step: calcHead", ch.toSMLString(), ((CalcHead) test)
	// .toSMLString());
			System.out.println(".end step_b-----");
			//end step_b
			// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
			//let calculate the first formula
			Tactic tac = ct.fetchProposedTactic();
			//sent to bridge: fetchProposedTactic 1;
			System.out.println(".2nd tac:" + tac.toSMLString());
			assertEquals("mini-step: 2nd form", tac.toSMLString(),
					"Apply_Method [\"Test\",\"squ-equ-test-subpbl1\"]");
			ct.setNextTactic(tac);
			//sent to bridge: setNextTactic 1
			// (Apply_Method["Test","squ-equ-test-subpbl1"]);
			//sent to bridge: autoCalculate 5 (Step 1);
			//^^^implicit in setNextTactit; remove ^^^; see @@@
			//###ct.autoCalculate(IToCalc.SCOPE_CALCULATION,1);//decomment ########
	
			cce = ((MockIToUser) user).getCalcChangedEvent();
			assertEquals("unchanged", cce.getLastUnchangedFormula().toSMLString(),
					"([1],Frm)");
			//@@@@@error@@@error@@@error@@@error@@@error@@@----"([],Pbl)"
			assertEquals("deleted", cce.getLastDeletedFormula().toSMLString(),
					"([1],Frm)");
			//@@@@@error@@@error@@@error@@@error@@@error@@@----"([],Pbl)"
			assertEquals("generated", cce.getLastGeneratedFormula().toSMLString(),
					"([1],Frm)");
	
			Position pos = new Position();//delete with @@@@@error above
			pos.setKind("Pbl");//delete with @@@@@error above
			ci = new CalcIterator((CalcTree) ct, pos);//delete with @@@@@error above
			Vector elems = ct.getElementsFromTo(ci,//cce.getLastUnchangedFormula(),
					cce.getLastGeneratedFormula(), new Integer(2), false);
			// before getElementsFromTo:
			// CalcFormula fa = (CalcFormula)
	 (cce.getLastGeneratedFormula()).getFormula();
			// TODO introduce getElementsFromTo below in setNextTactic
			CalcFormula fa = (CalcFormula) (elems.firstElement());
			System.out.println(".2nd form: " + fa.toSMLString());
			assertEquals("mini-auto: 2nd form", fa.toSMLString(), "x + 1 = 2");
	
			System.out.println(".end step_d-----");
			//end step_c
			// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
			//let calculate the second formula
			tac = ct.fetchProposedTactic();
			System.out.println(".3rd tac: " + tac.toSMLString());
			assertEquals("mini-step: 3rd tac", tac.toSMLString(),
					"Rewrite_Set \"norm_equation\"");
	
			ct.setNextTactic(tac);
			//sent to bridge: autoCalculate 5 (Step 1);
			//^^^implicit in setNextTactit; remove ^^^; see @@@
			//##ct.autoCalculate(IToCalc.SCOPE_CALCULATION,1);//decomment ##
			cce = ((MockIToUser) user).getCalcChangedEvent();
			//@@@@@error@@@error@@@error@@@error@@@error@@@----"([1],Res)"
			assertEquals("unchanged", cce.getLastUnchangedFormula().toSMLString(),
					"([1],Res)");//"([1],Frm)");...correct, but see
			// CalcTree#setNextTactic
			//@@@@@error@@@error@@@error@@@error@@@error@@@----"([1],Res)"
			assertEquals("deleted", cce.getLastDeletedFormula().toSMLString(),
					"([1],Res)");//"([1],Frm)");...correct, but see
			// CalcTree#setNextTactic
			assertEquals("generated", cce.getLastGeneratedFormula().toSMLString(),
					"([1],Res)");
	
			fa = (CalcFormula) (cce.getLastGeneratedFormula()).getFormula();
			System.out.println(".3rd form: " + fa.toSMLString());
			assertEquals("mini-auto: 3rd form", fa.toSMLString(),
					"x + 1 + -1 * 2 = 0");
	
			System.out.println(".end step_e-----");
			//end step_d
			// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
			//let calculate the third formula
			tac = ct.fetchProposedTactic();
			assertEquals("mini-step: 4th tac", tac.toSMLString(),
					"Rewrite_Set \"Test_simplify\"");
	
			ct.setNextTactic(tac);
			cce = ((MockIToUser) user).getCalcChangedEvent();
			//@@@@@error@@@error@@@error@@@error@@@error@@@----"([2],Res)"
			assertEquals("unchanged", cce.getLastUnchangedFormula().toSMLString(),
					"([2],Res)");//"([1],Res)");...correct, but see
			// CalcTree#setNextTactic
			//@@@@@error@@@error@@@error@@@error@@@error@@@----"([2],Res)"
			assertEquals("deleted", cce.getLastDeletedFormula().toSMLString(),
					"([2],Res)");//"([1],Res)");...correct, but see
			// CalcTree#setNextTactic
			assertEquals("generated", cce.getLastGeneratedFormula().toSMLString(),
					"([2],Res)");
	
			fa = (CalcFormula) (cce.getLastGeneratedFormula()).getFormula();
			System.out.println(".4th form:" + fa.toSMLString());
			assertEquals("mini-auto: 4th form", fa.toSMLString(), "-1 + x = 0");
	
			System.out.println(".end step_f-----");
			//end step_e
			// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		}
		
		/**
		 * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl'; comprises
	 the
		 * usecase UC\label{SOLVE:INFO:intermediate-steps} see
		 * sml/systest/FE-interface.sml "interSteps: on 'miniscript with
	 mini-subpbl'"
		 */
		public void testIntermediateSteps() throws Exception {
			System.out.println("---BEGIN isac.bridge.TestBridgeInput#testIntermediateSteps");
			// ~~~~~~this should (repeatedly) be done in a separate method~~~~~~~~~
			//prepare Formalization for start calculation in step_b
			Formalization fmz = null;
			File f = new File(
					"isac/src/java-tests/isac/bridge/data/mini-auto-step",
					"step_a.fmz");
			try {
				ObjectInputStream ois = new ObjectInputStream(
						new FileInputStream(f));
				fmz = (Formalization) ois.readObject();
				ois.close();
			} catch (IOException e) {
				e.printStackTrace();
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
			}
			//start calculation
			MathEngine.init("localhost");
			MathEngine me = MathEngine.getMathEngine();
			IToCalc ct = me.startCalculation(fmz);
	
			IToUser user = new MockIToUser();
			ct.addDataChangeListener(user);
			CalcChangedEvent cce = null;
			ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 0);
			// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
			
			// intermediateSteps(([2],Res)) gives 6 new steps
			Position pos = new Position();//([2],Res)
			pos.addInt("2");pos.setKind("Res");
			ICalcIterator ci = new CalcIterator((CalcTree)ct, pos);
	
			ct.intermediateSteps(ci);
			cce = ((MockIToUser) user).getCalcChangedEvent();
			Vector elems = ct.getElementsFromTo(cce.getLastUnchangedFormula(),
					cce.getLastGeneratedFormula(), new Integer(3), false);
			ICalcElement fch = null;
			for (int i=0; i<elems.size(); i++) {
				fch = (ICalcElement) elems.elementAt(i);
				if (fch instanceof CalcFormula)
					System.out.println(".step "+i+": "+
	 ((CalcFormula)fch).getPosition().toSMLString()
							+ " "+fch.toSMLString());
				else
					System.out.println(".step "+i+": "+
	 ((CalcHead)fch).getPosition().toSMLString()
							+ " "+fch.toSMLString());
			}
			System.out.println(".-------");
			assertEquals("mini-auto: steps= ",elems.size(),7);
			assertEquals("mini-auto: first step",
					((ICalcElement) elems.firstElement()).toSMLString(), "x + 1 + -1 * 2 = 0");
			assertEquals("mini-auto: last step",
					((ICalcElement) elems.lastElement()).toSMLString(), "-1 + x = 0");
			
			// intermediateSteps(([3,2],Res)) gives 3 new steps
			pos = new Position();//([3,2],Res)
			pos.addInt("3");pos.addInt("2");pos.setKind("Res");
			ci = new CalcIterator((CalcTree)ct, pos);
	
			ct.intermediateSteps(ci);
			cce = ((MockIToUser) user).getCalcChangedEvent();
			elems = ct.getElementsFromTo(cce.getLastUnchangedFormula(),
					cce.getLastGeneratedFormula(), new Integer(3), false);
			fch = null;
			for (int i=0; i<elems.size(); i++) {
				fch = (ICalcElement) elems.elementAt(i);
				if (fch instanceof CalcFormula)
					System.out.println(".step "+i+": "+
	 ((CalcFormula)fch).getPosition().toSMLString()
							+ " "+fch.toSMLString());
				else
					System.out.println(".step "+i+": "+
	 ((CalcHead)fch).getPosition().toSMLString()
							+ " "+fch.toSMLString());
			}
			System.out.println(".-------");
			assertEquals("mini-auto: steps= ",elems.size(),3);
			assertEquals("mini-auto: first step",
					((ICalcElement) elems.firstElement()).toSMLString(), "x = 0 + -1 * -1");
			assertEquals("mini-auto: last step",
					((ICalcElement) elems.lastElement()).toSMLString(), "x = 1");
			
			// intermediateSteps(([3],Res)) on CalcHead gives NO new steps
			pos = new Position();//([3],Res)
			pos.addInt("3");pos.setKind("Res");
			ci = new CalcIterator((CalcTree)ct, pos);
	
			ct.intermediateSteps(ci);
			cce = ((MockIToUser) user).getCalcChangedEvent();
			elems = ct.getElementsFromTo(cce.getLastUnchangedFormula(),
					cce.getLastGeneratedFormula(), new Integer(1), false);
			fch = null;
			for (int i=0; i<elems.size(); i++) {
				fch = (ICalcElement) elems.elementAt(i);
				if (fch instanceof CalcFormula)
					System.out.println(".step "+i+": "+
	 ((CalcFormula)fch).getPosition().toSMLString()
							+ " "+fch.toSMLString());
				else
					System.out.println(".step "+i+": "+
	 ((CalcHead)fch).getPosition().toSMLString()
							+ " "+fch.toSMLString());
			}
			System.out.println(".-------");
			assertEquals("mini-auto: steps= ",elems.size(),3);
			assertEquals("mini-auto: first step",
					((ICalcElement) elems.firstElement()).toSMLString(), "-1 + x = 0");
			assertEquals("mini-auto: last step",
					((ICalcElement) elems.lastElement()).toSMLString(), "x = 1");
			
			// intermediateSteps(([],Res)) gives whole calctree (on 1st level)
			pos = new Position();//([],Res)
			pos.setKind("Res");
			ci = new CalcIterator((CalcTree)ct, pos);
	
			ct.intermediateSteps(ci);
			cce = ((MockIToUser) user).getCalcChangedEvent();
			elems = ct.getElementsFromTo(cce.getLastUnchangedFormula(),
					cce.getLastGeneratedFormula(), new Integer(1), false);
			fch = null;
			for (int i=0; i<elems.size(); i++) {
				fch = (ICalcElement) elems.elementAt(i);
				if (fch instanceof CalcFormula)
					System.out.println(".step "+i+": "+
	 ((CalcFormula)fch).getPosition().toSMLString()
							+ " "+fch.toSMLString());
				else
					System.out.println(".step "+i+": "+
	 ((CalcHead)fch).getPosition().toSMLString()
							+ " "+fch.toSMLString());
			}
			assertEquals("mini-auto: steps= ",elems.size(),6);
			assertEquals("mini-auto: first step",
					((ICalcElement) elems.firstElement()).toSMLString(), "x + 1 = 2");
			assertEquals("mini-auto: last step",
					((ICalcElement) elems.lastElement()).toSMLString(), "[x = 1]");
		}
	
		/**
		 * uses the example 'IsacCore/Tests/minipbl_with_mini-subpbl'; comprises
		 * the UC\label{SOLVE:HIDE:getTactic} and
	   * UC\label{SOLVE:MANUAL:TACTIC:listall}
		 * see sml/systest/FE-interface.sml '- getTactic, fetchApplicableTactics -'
		 */
		public void testTactics() throws Exception {
			System.out.println("---BEGIN isac.bridge.TestBridge#testTactics");
			// ~~~~~~this should (repeatedly) be done in a separate method~~~~~~~~~
			//prepare Formalization for start calculation in step_b
			Formalization fmz = null;
			File f = new File(
					"isac/src/java-tests/isac/bridge/data/mini-auto-step",
					"step_a.fmz");
			try {
				ObjectInputStream ois = new ObjectInputStream(
						new FileInputStream(f));
				fmz = (Formalization) ois.readObject();
				ois.close();
			} catch (IOException e) {
				e.printStackTrace();
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
			}
			//start calculation
			MathEngine.init("localhost");
			MathEngine me = MathEngine.getMathEngine();
			IToCalc ct = me.startCalculation(fmz);
	
			IToUser user = new MockIToUser();
			ct.addDataChangeListener(user);
			CalcChangedEvent cce = null;
			ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 0);
			// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
			// on (root) CalcHead: also with getApplicableTactics only one Tactic
			Position pos = new Position();//([],Pbl)
			pos.setKind("Pbl");
			ICalcIterator ci = new CalcIterator((CalcTree) ct, pos);
	
			Tactic tac = ci.getTactic();// <--------------------------------
			assertEquals("at ([],Pbl) found: ", ((Tactic) tac).toSMLString(),
					"Apply_Method [\"Test\",\"squ-equ-test-subpbl1\"]");
	
			Vector tacs = ci.getApplicableTactics(99999);// <---------------
			assertEquals(((Tactic) tacs.firstElement()).toSMLString(),
					"Apply_Method [\"Test\",\"squ-equ-test-subpbl1\"]");
			assertEquals("number of appl. tacs = ", tacs.size(), 1);
	
			// somewhere in rootproblem: all Tactics in the method
			pos = new Position();//([1],Res)
			pos.addInt("1");
			pos.setKind("Res");
			ci = new CalcIterator((CalcTree) ct, pos);
	
			tac = ci.getTactic();// <---------------------------------------
			assertEquals(tac.toSMLString(), "Rewrite_Set \"Test_simplify\"");
	
			tacs = ci.getApplicableTactics(99999);// <----------------------
			System.out.println(".at " + ci.toSMLString() + ":");
			for (int i = 0; i < tacs.size(); i++)
				System.out.println(".." + ((Tactic) tacs.get(i)).toSMLString());
			assertEquals("number of appl. tacs = ", tacs.size(), 4);
	
			// on CalcHead of subproblem: only Apply_Method
			pos = new Position();//([3],Pbl)
			pos.addInt("3");
			pos.setKind("Pbl");
			ci = new CalcIterator((CalcTree) ct, pos);
	
			tac = ci.getTactic();// <---------------------------------------
			assertEquals(tac.toSMLString(),
					"Apply_Method [\"Test\",\"solve_linear\"]");
	
			tacs = ci.getApplicableTactics(99999);// <----------------------
			assertEquals("at ([3],Pbl) found: ", ((Tactic) tacs.firstElement())
					.toSMLString(), "Apply_Method [\"Test\",\"solve_linear\"]");
			assertEquals("number of appl. tacs = ", tacs.size(), 1);
	
			// somewhere in subproblem: all Tactics in this (other) method
			pos = new Position();//([3,1],Res)
			pos.addInt("3");
			pos.addInt("1");
			pos.setKind("Res");
			ci = new CalcIterator((CalcTree) ct, pos);
	
			tac = ci.getTactic();// <---------------------------------------
			assertEquals(tac.toSMLString(),"Rewrite_Set \"Test_simplify\"");
	
			tacs = ci.getApplicableTactics(99999);// <----------------------
			System.out.println(".at " + ci.toSMLString() + ":");
			for (int i = 0; i < tacs.size(); i++)
				System.out.println(".." + ((Tactic) tacs.get(i)).toSMLString());
			assertEquals("at ([3,1],Res) found: ", tacs.size(), 2);
	
			// on the result of the subproblem again all tacs of the rootpbl
			pos = new Position();//([3],Res)
			pos.addInt("3");
			pos.setKind("Res");
			ci = new CalcIterator((CalcTree) ct, pos);
	
			tac = ci.getTactic();// <---------------------------------------
			assertEquals(tac.toSMLString(),"Check_elementwise \"Assumptions\"");
	
			tacs = ci.getApplicableTactics(99999);// <----------------------
			System.out.println(".at " + ci.toSMLString() + ":");
			for (int i = 0; i < tacs.size(); i++)
				System.out.println(".." + ((Tactic) tacs.get(i)).toSMLString());
			assertEquals("at ([3],Res) found: ", tacs.size(), 4);
	
			// but on the final result there are _NO_ tacs applicable
			pos = new Position();//([],Res)
			pos.setKind("Res");
			ci = new CalcIterator((CalcTree) ct, pos);
			// tac = ci.getTactic();// <------------------------------------
			// tacs = ci.getApplicableTactics(99999);// <-------------------
			// TODO handle error msg
		}
		
	/**
	 * uses the example 'IsacCore/Tests/rat.equ, asms'; comprises the
	 * usecases UC\label{SOLVE:HELP:assumptions}
	 * and *UC\label{SOLVE:HELP:assumptions-origin}
	 * sml/systest/FE-interface.sml '- getAssumptions, getAccumulatedAsms -' 
	 */
	public void testAssumptions() throws Exception {
		System.out.println("---BEGIN isac.bridge.TestBridge#testAssumptions");

		//prepare Formalization for start calculation
		Formalization fmz = null;
		KEStoreServices ks = new KEStoreServices(
				"isac/src/java-tests/isac/kestore/data");
		Vector vec = ks.loadContent("exp", "exp_IsacCore_Tests_1c.xml");
		String xmlstr = (String) vec.firstElement();
		FormalizationDigest fd = new FormalizationDigest();
		fmz = fd.parseFormalization((String) xmlstr);

		//start and perform calculation
		MathEngine.init("localhost");
		MathEngine me = MathEngine.getMathEngine();
		IToCalc ct = me.startCalculation(fmz);

		IToUser user = new MockIToUser();
		ct.addDataChangeListener(user);
		CalcChangedEvent cce = null;
		ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 0);

		//		for (int i = 0; i < forms.size(); i++)
		//			System.out.println(".asm= "
		//					+ ((Formula) forms.elementAt(i)).toSMLString());

		// here the assumption arises
		Position pos = new Position();//([3],Res)
		pos.addInt("3");
		pos.setKind("Res");
		CalcIterator ci = new CalcIterator((CalcTree) ct, pos);

		Assumptions asm = ci.getAssumptions();//<---------------------------
		Vector forms = asm.getFormulae();
		assertEquals("asm generated here:", ((Formula) forms.firstElement())
				.toSMLString(), "9 * x + (x ^^^ 3 + -6 * x ^^^ 2) ~= 0");
		asm = ci.getAccumulatedAssumptions();//<----------------------------
		forms = asm.getFormulae();
		assertEquals("is the only asm:", ((Formula) forms.firstElement())
				.toSMLString(), "9 * x + (x ^^^ 3 + -6 * x ^^^ 2) ~= 0");

		// here the assumption is checked
		pos = new Position();//([5],Res)
		pos.addInt("5");
		pos.setKind("Res");
		ci = new CalcIterator((CalcTree) ct, pos);

		asm = ci.getAssumptions();//<---------------------------------------
		forms = asm.getFormulae();
		assertEquals("here no asm generated:", forms.size(), 0);
		asm = ci.getAccumulatedAssumptions();//<----------------------------
		forms = asm.getFormulae();
		assertEquals("acumulated from above:", ((Formula) forms.firstElement())
				.toSMLString(), "9 * x + (x ^^^ 3 + -6 * x ^^^ 2) ~= 0");

		System.out.println("---END isac.bridge.TestBridge");
	}

}