/*
 * Created on Sep 11, 2003
 */
package isac.bridge;

import isac.util.formulae.Assumptions;
import isac.util.formulae.CalcHead;
import isac.util.formulae.CalcFormula;
import isac.util.formulae.Position;
import isac.util.interfaces.ICalcElement;
import isac.util.interfaces.ICalcIterator;
import isac.util.tactics.Tactic;

import java.io.Serializable;
import java.rmi.RemoteException;
import java.util.Vector;

import org.apache.log4j.Logger;

/**
 * Iterator class to navigate through a calcTree. CalcIterator implements
 * several move Methods to navigate up/down a CalcTree, go into Subproblems,..
 * One can retrive also the element in the CalcTree currently marked by this
 * Iterator.
 * 
 * @author Richard Gradischnegg
 */
public class CalcIterator implements ICalcIterator, Serializable {

	private CalcTree calc_tree_;

	private int calc_treeID_;//from SML + mapped by smlCalcIDtoJavaCalcID(int);

	private int iteratorID_;//from SML TODO.WN041116 drop with new Iterator

	//WN041209 i.e. + SML: make Iterator n 1 implicit to CalcTree n.

	private Position sml_pos_;//unique within the sml calctree

	static Logger logger_ = Logger.getLogger(CalcIterator.class.getName());

	/**
	 * Construct a new calculation element iterator object attached to the
	 * passed calculation tree
	 * 
	 * @param calcTree
	 *            calculation tree
	 */
	protected CalcIterator(CalcTree calcTree) {
		logger_.debug("CalcIterator(new Position)");
		this.setSmlPos(new Position());
		this.calc_tree_ = calcTree;
		this.calc_treeID_ = calcTree.getId();
		//		try {
		//			this.iteratorID_ = calcTree.getMathEngine().iterator(calc_treeID_);
		//		} catch (RemoteException e) {
		//			// TODO Auto-generated catch block
		//			e.printStackTrace();
		//	    }
		this.iteratorID_ = 4711;//this selects for changing the CalcTree
	}

	/**
	 * Construct a new calculation element iterator object attached to the
	 * passed calculation tree with initial position (fetched from a formula on
	 * the worksheet)
	 * 
	 * @param calcTree
	 *            calculation tree
	 */
	protected CalcIterator(CalcTree calcTree, Position sml_pos) {
		logger_.debug("CalcIterator(" + sml_pos.toSMLString() + ")");
		this.setSmlPos(sml_pos);
		this.calc_tree_ = calcTree;
		this.calc_treeID_ = calcTree.getId();
		this.iteratorID_ = 4711;//this is calculated by the KE without state
		//		try {
		//			this.iteratorID_ = calcTree.getMathEngine().iterator(calc_treeID_);
		//		} catch (RemoteException e) {
		//			// TODO Auto-generated catch block
		//			e.printStackTrace();
		//		}
	}

	/**
	 * Turn this iterator into a HotSpot iterator i.e. use it to move the active
	 * Formula. This operation is not reversible.
	 * 
	 * @deprecated since autoCalculate etc. is applied to _CalcTree_ and all
	 *             other CalcIterators just read
	 */
	void makeHotSpot() {
		this.iteratorID_ = 1;
	}

	/**
	 * Reference the first (root) element in the attached calculation tree.
	 * 
	 * @return Like all the moveXXX methods this returns true on success, false
	 *         if there is no element to move to.
	 */
	public boolean moveRoot() {
		logger_.fatal(" DG->BR: moveRoot()");
		try {
			//WN041208 return calc_tree_.getMathEngine().moveRoot(calc_treeID_,
			// iteratorID_);
			Position p = calc_tree_.getMathEngine().moveRoot(calc_treeID_,
					iteratorID_);
			if (p == null) {
				logger_.fatal(" DG<-BR: moveRoot() <- false NULL###");
				return false;
			} else {
				this.setSmlPos(p);
				logger_.fatal(" DG<-BR: moveRoot() <- true");
				return true;
			}
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		logger_.fatal(" DG<-BR: moveRoot() <- false RemoteException###");
		return false;
	}

	/**
	 * Reference the previous formula in the attached calculation tree.
	 * 
	 * @return Like all the moveXXX methods this returns true on success, false
	 *         if there is no element to move to.
	 */
	public boolean moveUp() {
		logger_.fatal(" DG->BR: moveUp()");
		try {
			//WN041208 return calc_tree_.getMathEngine().moveUp(calc_treeID_,
			// iteratorID_, sml_pos_);
			Position p = calc_tree_.getMathEngine().moveUp(calc_treeID_,
					iteratorID_, sml_pos_);
			if (p == null) {
				logger_.fatal(" DG<-BR: moveUp() <- false NULL###");
				return false;
			} else {
				this.setSmlPos(p);
				logger_.fatal(" DG<-BR: moveUp() <- true");
				return true;
			}
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		logger_.fatal(" DG<-BR: moveUp() <- false RemoteException###");
		return false;
	}

	/**
	 * Reference the next formula in the attached calculation tree.
	 * 
	 * @return Like all the moveXXX methods this returns true on success, false
	 *         if there is no element to move to.
	 */
	public boolean moveDown() {
		logger_.fatal(" DG->BR: moveDown()");
		try {
			//WN041208 return calc_tree_.getMathEngine().moveDown(calc_treeID_,
			// iteratorID_, sml_pos_);
			Position p = calc_tree_.getMathEngine().moveDown(calc_treeID_,
					iteratorID_, sml_pos_);
			if (p == null) {
				logger_.fatal(" DG<-BR: moveDown() <- false NULL###");
				return false;
			} else {
				this.setSmlPos(p);
				logger_.fatal(" DG<-BR: moveDown() <- true");
				return true;
			}
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		logger_.fatal(" DG<-BR: moveDown() <- false RemoteException###");
		return false;
	}

	/**
	 * Enter the next coarser level of refinement in the attached calculation
	 * tree.
	 * 
	 * @return Like all the moveXXX methods this returns true on success, false
	 *         if there is no element to move to.
	 */
	public boolean moveLevelUp() {
		logger_.fatal(" DG->BR: moveLevelUp()");
		try {
			//WN041208 return
			// calc_tree_.getMathEngine().moveLevelUp(calc_treeID_, iteratorID_,
			// sml_pos_);
			Position p = calc_tree_.getMathEngine().moveLevelUp(calc_treeID_,
					iteratorID_, sml_pos_);
			if (p == null) {
				logger_.fatal(" DG<-BR: moveLevelUp() <- false NULL###");
				return false;
			} else {
				this.setSmlPos(p);
				logger_.fatal(" DG<-BR: moveLevelUp() <- true");
				return true;
			}
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		logger_.fatal(" DG<-BR: moveLevelUp() <- false RemoteException###");
		return false;
	}

	/**
	 * Enter the next more detailed level of refinement in the attached
	 * calculation tree.
	 * 
	 * @return Like all the moveXXX methods this returns true on success, false
	 *         if there is no element to move to.
	 */
	public boolean moveLevelDown() {
		logger_.fatal(" DG->BR: moveLevelDown()");
		try {
			//WN041208 return
			// calc_tree_.getMathEngine().moveLevelDown(calc_treeID_,
			// iteratorID_, sml_pos_);
			Position p = calc_tree_.getMathEngine().moveLevelDown(calc_treeID_,
					iteratorID_, sml_pos_);
			if (p == null) {
				logger_.fatal(" DG<-BR: moveLevelDown() <- false NULL###");
				return false;
			} else {
				this.setSmlPos(p);
				logger_.fatal(" DG<-BR: moveLevelDown() <- true");
				return true;
			}
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		logger_.fatal(" DG<-BR: moveLevelDown() <- false RemoteException###");
		return false;
	}

	/**
	 * Reference the calcHead that belongs to the currently selected item
	 */
	public boolean moveCalcHead() {
		logger_.fatal(" DG->BR: moveCalcHead()");
		try {
			//WN041208return
			// calc_tree_.getMathEngine().moveCalcHead(calc_treeID_,
			// iteratorID_, sml_pos_);
			Position p = calc_tree_.getMathEngine().moveCalcHead(calc_treeID_,
					iteratorID_, sml_pos_);
			if (p == null) {
				logger_.fatal(" DG<-BR: moveCalcHead() <- false NULL###");
				return false;
			} else {
				this.setSmlPos(p);
				logger_.fatal(" DG<-BR: moveCalcHead() <- true");
				return true;
			}
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		logger_.fatal(" DG<-BR: moveCalcHead() <- false RemoteException###");
		return false;
	}

	/**
	 * Reference the tactic which resulted in the currently selected element of
	 * the attached calculation tree.
	 * <p>
	 * A special case arises if the tactic to continue the current calculation
	 * is solving a subproblem. The subproblem is a calculation on its own,
	 * represented by a separate calculation(sub-)tree object. A reference to
	 * the calculationtree representing the subproblem is passed to the
	 * worksheet. The worksheet will handle the situation accordingly, e.g. by
	 * opening a new worksheet or displaying the new calculationtree (indented)
	 * in the current worksheet.
	 * 
	 * @return Like all the moveXXX methods this returns true on success, false
	 *         if there is no element to move to.
	 * @deprecated
	 */
	public boolean moveTactic() {
		try {
			return calc_tree_.getMathEngine().moveTactic(calc_treeID_,
					iteratorID_);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return false;
	}

	/**
	 * Reference the formula which resulted in the currently selected element of
	 * the attached calculation tree.
	 * 
	 * @deprecated
	 */
	public boolean moveFormula() {
		try {
			return calc_tree_.getMathEngine().moveFormula(calc_treeID_,
					iteratorID_);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return false;
	}

	/*
	 * public Formula getFormula() { return new Formula(); }
	 * 
	 * public Tactic getTactic() { return new Tactic(); }
	 */

	/**
	 * The current level of nesting into subcalculations.
	 * 
	 * @return An int representing level of nesting (higher == more detailed)
	 *         <p>
	 *         Special values:
	 *         <p>
	 *         0: outmost level: only first and last step of calculation are on
	 *         this level
	 */
	public int getLevel() {
		int lev;
		Position pos = this.getPosition();
		if (pos != null) {
			lev = pos.getIntList().size();
			logger_.error(" DG<>BR: getLevel() <- " + lev);
			return lev;
		} else {
			logger_.error(" DG<>BR: getLevel() <- -1 NULL###");
			return -1;
		}
	}

	/**
	 * Returns the item that this iterator is currently referencing
	 * 
	 * @see isac.wsdialog.ICalcIterator#getElement()
	 * @deprecated in favour of get
	 */
	public ICalcElement getElement() {
		// TODO Auto-generated method stub
		ICalcElement result = null;
		try {
			result = calc_tree_.getMathEngine().getElement(calc_treeID_,
					sml_pos_);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return result;
	}

	/**
	 * Set the position of this iterator to the position of another, given
	 * iterator
	 * 
	 * @param calcIterator
	 *            the other iterator
	 * @deprecated WN041208 for reading sml_pos from kernel
	 *             (clone,moveActiveFormula)
	 */
	void moveTo(CalcIterator calcIterator) {
		this.moveRoot();
		Position otherPos = calcIterator.getPosition();
		//System.out.println(this.getPosition().toSMLString());
		//System.out.println(calcIterator.getPosition().toSMLString());
		while (!this.isOnPosition(otherPos)) {
			if (!this.moveDown())
				return;
			//System.out.println("clone:
			// this=="+this.getPosition().toSMLString());
			//System.out.println("clone:other=="+calcIterator.getPosition().toSMLString());
		}
	}

	// get the Position of the current element in the CalcTree
	public Position getPosition() {//TODO.WN041208 getSmlPos
		logger_.error(" DG<>BR: getPosition() <--" + sml_pos_.toSMLString());
		return sml_pos_;
		//		Position thisPos = null;
		//		ICalcElement ce = this.getElement();
		//		if (ce instanceof CalcHead) {
		//			thisPos = ((CalcHead) ce).getPosition();
		//		} else if (ce instanceof Formula) {
		//			thisPos = ((Formula) ce).getPosition();
		//		}
		//		return thisPos;
	}

	private void setSmlPos(Position p) {
		this.sml_pos_ = p;
	}

	/**
	 * check whether the iterator is on a given position
	 * 
	 * @deprecated
	 */
	private boolean isOnPosition(Position pos) {
		Position thisPos = this.getPosition();
		if (thisPos == null || pos == null)
			return false;
		if (!thisPos.equals(pos))
			return false;
		return true;
	}

	/**
	 * Clone this iterator. The result is a second, different iterator
	 * referencing the same item as this iterator
	 * 
	 * @see isac.wsdialog.ICalcIterator#clone()
	 */
	public Object clone() {
		logger_.debug("CalcIterator.clone() <-" + sml_pos_.toSMLString());
		CalcIterator clone = (CalcIterator) calc_tree_.iterator();
		clone.setSmlPos(sml_pos_);
		return clone;
	}

	/**
	 * Compare the referenced item of this iterator to the item of a second,
	 * given iterator
	 * 
	 * @return Integer
	 *         <p>
	 *         0 if the second iterator references the same item
	 *         <p>
	 *         -1 otherwise
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 *  
	 */
	public int compareTo(Object o) {
		int comp = this.sml_pos_.compareTo(((CalcIterator) o).sml_pos_);
		logger_.fatal(" DG<>BR: compareTo() <- " + comp);
		return comp;
	}

	public int getIteratorID() {
		return iteratorID_;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see isac.util.interfaces.ICalcIterator#isLast()
	 * @deprecated since CalcChangedEvent.last_generated_formula
	 */
	public boolean isLast() {
		// TODO: implement
		return true;
	}

	/**
	 * get the formula-part at a CalcIterator-position this can be either a
	 * Formula or a CalcHead
	 * 
	 * @see isac.util.interfaces.ICalcIterator#getFormula()
	 */
	public ICalcElement getFormula() {
		logger_.fatal(" DG->BR: getFormula()");
		ICalcElement fch = null;
		try {
			fch = calc_tree_.getMathEngine().getElement(calc_treeID_, sml_pos_);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		logger_.fatal(" DG<-BR: getFormula <- formula=" + fch.toSMLString());
		return fch;
	}

	/**
	 * @see isac.util.interfaces.ICalcIterator#getTactic()
	 */
	public Tactic getTactic() { //WN050217 kopiert von getFormula
								// !!!!!!!!!!!!!!!!!
		logger_.fatal(" DG->BR: getTactic()");
		Tactic tac = null;
		try {
			tac = calc_tree_.getMathEngine().getTactic(calc_treeID_, sml_pos_);
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		logger_.fatal(" DG<-BR: getTactic()" + tac.toSMLString());
		return tac;
	}

	/**
	 * @see isac.util.interfaces.ICalcIterator#getApplicableTactics()
	 */
	public Vector getApplicableTactics(int scope) throws RemoteException {
		logger_.fatal(" DG->BR: getApplicableTactics()");
		Vector tacs = null;
		try {
			tacs = calc_tree_.getMathEngine().getApplicableTactics(
					calc_treeID_, scope, sml_pos_);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		logger_.fatal(" DG<-BR: getApplicableTactics()");
		return tacs;
	}

	/**
	 * get the Assumptions generated by the Tactic applied to the Formula at a
	 * CalcIterator-position
	 * 
	 * @see isac.util.interfaces.ICalcIterator#getAssumptions()
	 */
	public Assumptions getAssumptions() {
		logger_.fatal(" DG->BR: getAssumptions()");
		Assumptions asms = null;
		try {
			asms = calc_tree_.getMathEngine().getAssumptions(calc_treeID_,
					sml_pos_);
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		logger_.fatal(" DG<-BR: getAssumption() <- " + asms.toSMLString());
		return asms;
	}

	/**
	 * @see isac.util.interfaces.ICalcIterator#getAccumulatedAssumptions()
	 */
	public Assumptions getAccumulatedAssumptions() throws RemoteException {
		Assumptions asms = null;
		try {
			asms = calc_tree_.getMathEngine().getAccumulatedAssumptions(
					calc_treeID_, sml_pos_);
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		logger_.fatal(" DG<-BR: getAssumption() <- " + asms.toSMLString());
		return asms;
	}

	public String toSMLString() {
		if (sml_pos_ == null)
			return "([],Und)";
		else
			return sml_pos_.toSMLString();
	}

}