/*
 * Created on Aug 13, 2003
 */
package isac.bridge;

/**
 * @author rgradisc
 */
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.Socket;
import java.util.Vector;

/**
 * This class handles the output of the clients
 * @author richard
 *
 */
class ClientOutputWorker implements Runnable {
  private Socket client;
  private int id;
  private ClientList clientList;  
  private BridgeMain bridge; 

  private BufferedReader clientIn;
  private PrintWriter clientOut;
  //private PrintWriter smlWriter;
  private Vector timeOutTimes;
  //---------------------------------------------------------------  
  public ClientOutputWorker(int id, Socket client, BridgeMain bridge) {
    this.id = id;
    this.client = client;
    this.bridge = bridge;
    this.timeOutTimes = bridge.getTimeOutTimes();
    this.clientList = bridge.getClientList();
    //this.smlWriter = bridge.getSmlWriter();    
    try {
      clientIn = new BufferedReader(new InputStreamReader(client.getInputStream()));
      clientOut = new PrintWriter(client.getOutputStream(), true);
      bridge.log(1, ">>> new client in : "+clientIn.toString()); 
	  bridge.log(1, ">>> new client out: "+clientOut.toString());
    } catch (IOException e) {
      System.out.println("in or out failed");
      System.exit(-1);
    }
    //register client in client list
    synchronized(clientList) {	
	  clientList.addClient(id, clientOut);
    }    
  }

  //---------------------------------------------------------------
  public void run() {
    String line;
    while (true) {
      try {    
      	  	
        line = clientIn.readLine();                
        if (line == null) {
          clientList.removeClient(id);          
          break; // exit mainloop -> exit thread 
        }
        sendData(line);
      } catch (IOException e) {
        bridge.log(1, "Read failed");
        break;
        //System.exit(-1);
      }
    }
    bridge.log(1, "[[from client" + id + ": connection closed]]");    
    bridge.getClientList().removeClient(id);
    try {
      client.shutdownInput();
      client.shutdownOutput();
    } catch (IOException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
  }

  //---------------------------------------------------------------
  private synchronized void sendData(String line) {    
    //so far: only one kernel 
	bridge.log(1, "[[from client" + id + ": " + line);       
    //bridge.bufferInput(1, id, line); // (kernelID, clientID, line)          
    timeOutTimes.add(new Long(System.currentTimeMillis())); // getTimeOutTimes(kernelID)
	bridge.getSmlWriter().println(line); // getSmlWriter(kernelID) 
	 
  }
}
