/****************************************************************** 
* Copyright (c) 2003, Mario Hochreiter, Student of 
* Software Engineering at FH-Hagenberg, Austria. 
* Project member of the isac team at the Institute for 
* Software-Technologie, Graz University of Technology, Austria. 
* 
* Use is subject to license terms.
******************************************************************/

package isac.gui.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * @author Mario Hochreiter
 * @version 1.0
 *
 */
public class Hierarchy {
  /** root node of the whole hierarchy */
  protected HierarchyNode root;
  /** holds all nodes which have child elements */
  protected Map rootNodes;
  /** holds the child nodes */
  protected List childNodes;
  
  // needed create the effect of a hierarchy when writing to stdout
  protected String LEVELSTRING = "--";
  
  public Hierarchy() {
    rootNodes = new HashMap();
  }
  
  public void addNode(HierarchyNode parent, HierarchyNode child) {
    if (parent == null) {
      root = child;
      rootNodes.put(root, null);
    } else {
      List childs = this.getChilds(parent);
      if (childs == null) {
        childs = new ArrayList();
        childs.add(child);
        rootNodes.put(parent, childs);        
      } else {
        childs.add(child);
      }      
    }
  }

  public List getChilds(Object parent) {
    return (List)rootNodes.get(parent);
  }
  
  public HierarchyNode getChild(HierarchyNode parent, int index) 
                              throws IllegalArgumentException {
    HierarchyNode childNode = null;
    List childs = (List)rootNodes.get(parent);
    if (childs != null) {
      childNode = (HierarchyNode)childs.get(index);
      return childNode;
    } else {
      throw new IllegalArgumentException("Parent node has nod child!");
    }    
  }
  
  public int getIndexOfChild(HierarchyNode parent, HierarchyNode child) 
                            throws IllegalArgumentException {
    List childs = (List)rootNodes.get(parent);
    if (childs != null) {
      return childs.indexOf(child);
    } else {
      throw new IllegalArgumentException("Parent node has no child!");
    }
  }
  
  public HierarchyNode getRoot() {
    return root;
  }
  
  private String printNodesHierarchy(HierarchyNode hNode, int level) {
    StringBuffer sb = new StringBuffer();
    
    if (level == 0) {
      sb.append(hNode.id)
      .append(" ")
      .append(hNode.contentref)
      .append("\n");
    }
    
    List childs = this.getChilds(hNode);
    if (childs != null) {
      level++;
      Iterator it = childs.iterator();
      while (it.hasNext()) {
        HierarchyNode node = (HierarchyNode)it.next();        
        StringBuffer levelBuffer = new StringBuffer();
        for (int i = 0; i < level; i++)
          levelBuffer.append(LEVELSTRING);
          
        sb.append("+").append(levelBuffer).append(": ")
          .append(node.id)
          .append(" ")
          .append(node.contentref)
          .append("\n")
          .append(printNodesHierarchy(node, level));
      }
    }
    return sb.toString();
  }
  
  public String toString() {
    return printNodesHierarchy(this.getRoot(), 0);
  }
}
