/****************************************************************** 
* Copyright (c) 2003, Mario Hochreiter, Student of 
* Software Engineering at FH-Hagenberg, Austria. 
* Project member of the isac team at the Institute for 
* Software-Technologie, Graz University of Technology, Austria. 
* 
* Use is subject to license terms.
******************************************************************/

package isac.gui.util;

import java.io.StringReader;

import org.apache.xerces.parsers.DOMParser;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;

/**
 * Abstract base class for all XMLHierarchy parser (like ExampleParser, ...).
 * 
 * @author Mario Hochreiter
 * @version 1.0
 *
 */
public abstract class XMLHierarchyParser {
  
  protected static final String TAG_NODE = "NODE";
  // the different tags a node tag can have
  protected static final String TYPE_ID = "ID";

  protected static final String TYPE_CONTENTREF = "CONTENTREF";
  
  /** XML in String representation */
  protected String xml;
  /** XML as document object */
  protected Document document;
  /** Hierarchy that manages the different nodes (example, problem, method) */
  protected Hierarchy hiearchy;
  

  public XMLHierarchyParser(String xmlString) {
  	
  	System.out.println("WN##### XMLHierarchyParser: "+xmlString);
  	
    this.xml = xmlString;
    this.document = this.parseXMLStringDOM();
    this.hiearchy = new Hierarchy();
  }
  
  private Document parseXMLStringDOM() {
    InputSource is = new InputSource(new StringReader(xml));
    DOMParser parser = new DOMParser();
    try {
      parser.parse(is);
    } catch (Exception e) {
      e.printStackTrace();
    }
    return parser.getDocument();
  }
  
  public Node getRoot() {
    return document.getDocumentElement();
  }
}
