/****************************************************************** 
* Copyright (c) 2003, Mario Hochreiter, Student of 
* Software Engineering at FH-Hagenberg, Austria. 
* Project member of the isac team at the Institute for 
* Software-Technologie, Graz University of Technology, Austria. 
* 
* Use is subject to license terms.
******************************************************************/

package isac.gui.util;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.Text;

/**
 * Parses the XML representation of the problem hierarchy and converts
 * it to a object representation.
 * Each NODE Tag is converted to a <code>ProblemNode</code> object. The
 * problem nodes are managed by the <code>Hierarchy</code>.
 * 
 * @author Mario Hochreiter
 * @version 1.0
 *
 */
public class XMLProblemParser extends XMLHierarchyParser {
  
  public XMLProblemParser(String xml) {
    super(xml);
  }
  
  private void createProblemNodesHierarchy(Node node, ProblemNode root, 
                                           String txtType) {
    ProblemNode pblNode = null;
    String elemName = null;
    // is there anything to do?
    if (node == null) {
      return;
    }

    int type = node.getNodeType();
    switch (type) {
      case Node.DOCUMENT_NODE : {
          Document document = (Document)node;
          createProblemNodesHierarchy(document.getDocumentElement());
          break;
      }

      case Node.ELEMENT_NODE :  {
          String nodeName = node.getNodeName().trim(); 
          if (nodeName.equals(TAG_NODE)) {
            pblNode = new ProblemNode();
            hiearchy.addNode(root, pblNode);
            // System.out.println(node.getNodeName().trim()); 
          } else if (nodeName.equals(TYPE_ID)){
            elemName = TYPE_ID;
          } else if (nodeName.equals(TYPE_CONTENTREF)) {
            elemName = TYPE_CONTENTREF;
          }
          
          // drop through to entity reference
      }

      case Node.ENTITY_REFERENCE_NODE :  {
          Node child = node.getFirstChild();
          while (child != null) {
            // elemName is set -> next node is of type text
            if (elemName != null)
              createProblemNodesHierarchy(child, root, elemName);
            createProblemNodesHierarchy(child, pblNode, txtType);
            child = child.getNextSibling();
          }
          break;
          
      }

      case Node.CDATA_SECTION_NODE : {
        break;
      }

      case Node.TEXT_NODE :  {
          // unknown elemType or whitespace is ignored
          if (txtType == null)
            return;
             
          Text text = (Text)node;
          String txt = text.getNodeValue().trim();
          
          if (txtType.equals(TYPE_ID)) {
            root.setId(txt);
            // System.out.println("ID: " + txt);
          } else if (txtType.equals(TYPE_CONTENTREF)) {
            root.setContentref(txt);
            // System.out.println("CONTENTREF: " +txt); 
          }
          break;
      }
    }
  }
  
  public Hierarchy createProblemNodesHierarchy(Node node) {
    this.createProblemNodesHierarchy(node, null, null);
    return hiearchy;
  }
}
