/****************************************************************** 
* @author Alois Kirchsteiger member of the isac team
* Copyright (c) 2004 by Alois Kirchsteiger
* create 02.09.2004, 09:25:44
* Insitute for Softwaretechnology, Graz University of Technology, Austria. 
* 
* Use is subject to license terms.
******************************************************************/

//TODO: implement!!!

package isac.session;

import isac.util.formulae.CalcElement;

import java.io.Serializable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.Vector;

/**
 * @author Alois Kirchsteiger 02.09.2004 09:25:44
 */
public class UserModel implements Serializable {
  
   /**
    * successInfos stores the successInfos for CalcElements.
    * that means the key for this TreeMap is a calcElement, 
    * the value is the relating SuccessInfo
    */
    private TreeMap successInfos;
  
    public UserModel() {
    }
    
    /**
     * gets the info for each CalcElement of the parameter calcElements
     * @param calcElements this Vector must not contain anything else
     *        than CalcElements
     * @return a TreeMap with the SuccessInfo for each CalcElement in the
     *         calcElements Vector. Those CalcElements are the index in the
     *         HashMap, the relating SuccessInfos are the values
     */
    public Map getSuccessInfoFor(Vector calcElements) {
      TreeMap result = new TreeMap();
      Iterator iterator = calcElements.iterator();
      while (iterator.hasNext()) {
        CalcElement calcElement = (CalcElement)iterator.next();
        if (successInfos.containsKey(calcElement)) {
          result.put(calcElement, successInfos.get(calcElement));
        } else {
          // no successInfo available
          result.put(calcElement, null);
        }
      }
      //TODO
      return result;
    }
    
    /**
     * @param calcElement
     * @return SuccessInfo or null if no SuccessInfo available
     */
    public SuccessInfo getSuccessInfoFor(CalcElement calcElement) {
      return (SuccessInfo)successInfos.get(calcElement);
    }
    
    /**
     * @param calcElementsSuccesses, a TreeMap with a CalcElement
     *        as key and the relating success (Float between 0 and 1)
     *        as value
     */
    public void addSuccessInfoFor(Map calcElementsSuccesses) throws ClassCastException{
      Set set = calcElementsSuccesses.keySet();
      Iterator iterator = set.iterator();
      while (iterator.hasNext()) {
        try {
          CalcElement calcElement = (CalcElement)iterator.next();
          Float success = (Float)calcElementsSuccesses.get(calcElement);
          addSuccessInfoFor(calcElement, success.floatValue());
        } catch (ClassCastException e) {
          throw new ClassCastException("the parameter for the method " +
                                       "void addSuccessInfoFor(TreeMap calcElementsSuccesses) " +
                                       "must have CalcElements as keys and Floats as value!!");
        }
      }
    }
    
    public void addSuccessInfoFor(CalcElement calcElement, float success) {
      SuccessInfo successInfo;
      if (successInfos.containsKey(calcElement)) {
        successInfo = (SuccessInfo) successInfos.get(calcElement);
        successInfo.addSuccess(success);
        successInfos.put(calcElement, successInfo);
      } else {
        successInfo = new SuccessInfo(success);
        successInfos.put(calcElement, successInfo);
      }
    }
    
}
