/****************************************************************** 
* Copyright (c) 2003, Mario Hochreiter, Student of 
* Software Engineering at FH-Hagenberg, Austria. 
* Project member of the isac team at the Institute for 
* Software-Technologie, Graz University of Technology, Austria. 
* 
* Use is subject to license terms.
******************************************************************/

package isac.wsdialog;

import java.rmi.Remote;
import java.rmi.RemoteException;

import isac.bridge.CalcTree;
import isac.util.*;
import isac.util.formulae.*;
import isac.util.interfaces.*;



/**
 * @author Alan Krempler
 *
 */
/**
 * @author Alan Krempler
 * 
 * The interface of the Dialog Guide as seen from the Presentation Layer
 *
 */
public interface DGuide extends IUIElement, IToCalc, Remote {
  
  /**
   * To express that the user started a calculation from scratch
   */
	public static final int STARTFROM_EMPTY = 0;
	/**
	  * To express that the user started a calculation from the example collection 
	  */
	public static final int STARTFROM_EXAMPLE = 1;
	/**
		* To express that the user started a calculation from the hierarchy of problems
		*/
	public static final int STARTFROM_PROBLEM = 2;
	public static final int STARTFROM_METHOD = 3; //WN040826

  /**
   * The Dialog is not doing anything at the moment
   */
	public static final int DIALOGPHASE_IDLE = 10;
	/**
	 * The Dialog guides the user through the specification of a problem
	 */
	public static final int DIALOGPHASE_SPECIFY = 11;
	/**
	 * The Dialog guides the user to the solution of a problem
	 */
	public static final int DIALOGPHASE_SOLVE = 12;
	/**
	 * The Dialog guides the user through the specification of a subproblem
	 */
	public static final int DIALOGPHASE_SPECIFY_SUBPROBLEM = 3;
  

  /**
   * Start the specification phase by providing initial values to the Dialog
   * On success, the Dialog is in DIALOGPHASE_SPECIFY
   * 
   * @param user_id The user 
	 * @param f Formalization from the example collection or null
	 * @param started_from STARTFROM_EMPTY, STARTFROM_EXAMPLE, STARTFROM_PROBLEMTYPE
 	 * @param requested_calchead_view
 	 * @return
 	 * @throws RemoteException
 	 * 
 	 * @see isac.util.Formalization
 	 * @see isac.util.formulae.CalcHead#setViewStyle(int)
 	 */
	public CalcTree startCalculation(int user_id, Formalization f, int started_from, 
                                                     int requested_calchead_view)
                                                         throws RemoteException;
  
  /**
   * Start the solving phase by providing a complete CalcHead
   * On success, the Dialog is in DIALOGPHASE_SOLVE
   * @param mod_spec CalcHead (Model and Specification) to start the calculation with
 	 * @throws RemoteException
 	 */
	public void newCalculation(CalcHead mod_spec) throws RemoteException;


	/**
	 * Register the UI to be controlled by the Dialog. There is only
	 * one such UI, any already-registered UI will be detached.
	 * @param listener UI which is giong to listen to the Dialog
	 * @return true on success, flase othewise
	 * @throws RemoteException
	 */
	public boolean registerUIControlListener(IToWorksheet listener) throws RemoteException;
  
 
	/**
	 * Notify the Dialog about events or requests triggered by the user
	 * @param action What's new from the UI
	 * @return true, if the event is being processed, false if processing is denied or impossible
	 * @throws RemoteException
	 * @throws DialogProtocolException
	 */
	public boolean notifyUserAction (IUserAction action) 
	  		throws RemoteException, DialogProtocolException;
  
}
