(* global variables needed in ME, DG, and updated in knowledge
   WN.8.01 from modspec.sml
*)

(* data for methods stored in 'methods'-database *)
type met = 
     (*defaults for Rewrite' (rls, rrls have their own data):*)
     {rew_ord'  : rew_ord',   (*for rules in Detail*)
      erls      : rls,        (*the eval_rls for cond. in rules*)
      srls      : rls,        (*for evaluating list expressions in scr*)
      prls      : rls,        (*for evaluating predicates in ppc*)
      crls      : rls,        (*for check_elemtwise*)
      nrls      : rls,        (*canonical simplifier specific for this met*)
      calc      : calc list,  (*040207: from calclist'in fun prep_met *)
      (*branch  : TransitiveB set in append_problem at generation ob pblobj
       FIXXXME.8.03: set branch from met in Apply_Method*)

      (* compare type pbt:*)
      ppc: ((string *      (*field*)
	     (term *       (*description*)
	      term))       (*id | struct-var*) 
		list),
      pre: term list,      (*preconditions*)
      (*script*)  
      scr: scr (*prep_met requires either script or string "empty_script"*)
	   };
val e_met = {rew_ord' = "e_rew_ord'": rew_ord',
	      erls = e_rls, srls = e_rls, prls = e_rls,
	      calc = [], crls = e_rls, nrls = e_rls,
	      (*asm_thm = []: thm' list,
	      asm_rls = []: rls' list,*)
	      ppc = []: (string * (term * term)) list,
	      pre = []: term list,
	      scr = EmptyScr: scr}:met;


(** problem-types stored in format for usage in specify  **)
(*25.8.01 ----
val pbltypes = ref ([(e_pblID,[])]:(pblID * ((string * (* field "#Given",..*)
			     (term *   (* description      *)
			      term))    (* id | struct-var  *)
			     list)
		    ) list);*)
type pbt_ = (string *  (* field "#Given",..*)
	      (term *   (* description      *)
	       term));   (* id | struct-var  *)
val e_pbt_ = ("#Undef", (e_term, e_term)):pbt_;
type pbt = 
     {thy   : theory,     (* which allows to compile that pbt
			  TODO: search generalized for subthy (ref.p.69*)
      cas   : term option,(*'CAS-command'*)
      prls  : rls,        (* for preds in where_*)
      where_: term list,  (* where - predicates*)
      ppc   : pbt_ list,
      met   : metID list}; (* methods solving the pbt*)
val e_pbt = {thy=ProtoPure.thy,cas=None,prls=Erls,
	     where_=[],ppc=[],met=[]}:pbt;
fun pbt2 (str, (t1, t2)) = 
    pair2str (str, pair2str (term2str t1, term2str t2));
fun pbt2str pbt = (strs2str o (map (linefeed o pbt2))) pbt;


(*.'a is for pbt | met.*)
(*WN.24.4.03 -"- ... type parameters; afterwards naming inconsistent*)
datatype 'a ptyp = 
	 Ptyp of string *   (*element within pblID*)
		 'a list *  (*several pbts with different domIDs/thy
			      TODO: select by subthy (isaref.p.69)
			      presently only _ONE_ elem*)
		 ('a ptyp) list;   (*the children nodes*)
val e_Ptyp = Ptyp ("e_pblID",[e_pbt],[]);
val e_Mets = Ptyp ("e_metID",[e_met],[]);

type ptyps = (pbt ptyp) list;
val ptyps = ref ([e_Ptyp]:ptyps);
type mets = (met ptyp) list;
val mets = ref ([e_Mets]:mets);


(**+ breadth-first search on hierarchy of problem-types +**)

type pblRD = pblID;(*pblID are Reverted _on calling_ the retrieve-funs*)
     (* eg. ["equations","univariate","normalize"] while
	    ["normalize","univariate","equations"] is the related pblID
      WN.24.4.03: also used for metID*)

fun get_py thy d _ [] = 
    raise error ("get_pbt not found: "^(strs2str d))
  | get_py thy d [k] ((Ptyp (k',[py],_))::pys) =
    if k=k' then py
    else get_py thy d ([k]:pblRD) pys
  | get_py thy d (k::ks) ((Ptyp (k',_,pys))::pys') =
    if k=k' then get_py thy d ks pys
    else get_py thy d (k::ks) pys';
(*> ptyps:= 
[Ptyp ("1",[("ptyp 1",([],[]))],
	[Ptyp ("11",[("ptyp 11",([],[]))],
		[])
	 ]),
 Ptyp ("2",[("ptyp 2",([],[]))],
	[Ptyp ("21",[("ptyp 21",([],[]))],
		[])
	 ])
 ];
> get_py SqRoot.thy ["1"] ["1"] (!ptyps);
> get_py SqRoot.thy ["2","21"] ["2","21"] (!ptyps);
         _REVERSE_  .......... !!!!!!!!!!*)

(*TODO: search generalized for subthy*)
fun get_pbt (*thy:theory TODO:subthy*) (pblID:pblID) =
    let val pblRD = rev pblID;
    in get_py  ProtoPure.thy pblID pblRD (!ptyps) end;
(* get_pbt thy ["1"];
   get_pbt thy ["21","2"];
   *)




(*WN.24.4.03
val methods = 
  ref ([]:(metID * met) list);*)

(*fun get_met metID  = (the (assoc (!methods, metID)))
  handle _ => error ("unknown method: "^(metID2str metID));*)
(*fun get_met (metID:metID) = 
    let val metRD = rev metID;
    in get_py  ProtoPure.thy metID metRD (!mets) end;*)
fun get_met (metID:metID) = get_py  ProtoPure.thy metID metID (!mets);


type spec = 
     domID * (*WN.12.03: is replaced by thy from get_met ?FIXME? in:
	      specify (Init_Proof..), nxt_specify_init_calc,
	      assod (.SubProblem...), stac2tac (.SubProblem...)*)
     pblID * 
     metID;
fun spec2str ((dom,pbl,met)(*:spec*)) = 
  "(" ^ (quote dom) ^ ", " ^ (strs2str pbl) ^ 
  ", " ^ (strs2str met) ^ ")";
(*> spec2str empty_spec;
val it = "(\"\", [], (\"\", \"\"))" : string *)
val empty_spec = (e_domID,e_pblID,e_metID):spec;
val e_spec = empty_spec;





(*.association list with cas-commands, for generating a complete calc-head.*)
type castab = 
     (term *         (*cas-command, eg. 'solve'*)
      (spec * 	     (*theory, problem, method*)
       		     (*the function generating a kind of formalization*)
       (term list -> (*the arguments of the cas-command, eg. (x+1=2, x)*)
	(term *      (*description of an element*)
	 term list)  (*value of the element (alway put into a list)*)
	    list)))  (*of elements in the formalization*)
	 list;       (*of cas-entries in the association list*)

val castab = ref ([]: castab);
